<?php
// $Id: drubuntu.drush.inc,v 1.1 2010/03/12 01:24:06 grugnog Exp $

/**
 * LAMP default package list.
 */
define('DRUSH_DRUBUNTU_LAMP_PACKAGES', '$(tasksel --task-packages lamp-server) curl php5-curl php5-xdebug php5-gd php-apc php5-mcrypt php5-dev php-pear libmagic-dev dnsmasq');

/**
 * PECL default package list.
 */
define('DRUSH_DRUBUNTU_PECL_PACKAGES', 'uploadprogress fileinfo');

/**
 * Eclipse default package list.
 */
define('DRUSH_DRUBUNTU_ECLIPSE_PACKAGES', 'eclipse eclipse-pdt eclipse-plugin-cvs eclipse-subclipse');

/**
 * Webdev default package list.
 */
define('DRUSH_DRUBUNTU_WEBDEV_PACKAGES', 'cvs subversion svn-load git-core bzr meld mysql-query-browser mysql-admin phpmyadmin ghex siege gworldclock konqueror wine cabextract opera google-chrome-beta');

/**
 * IEs4Linux default URI.
 */
define('DRUSH_DRUBUNTU_IES4LINUX_URI', 'http://www.tatanka.com.br/ies4linux/downloads/ies4linux-2.99.0.1.tar.gz');

/**
 * Firefox addons default IDs (from addons.mozilla.org).
 * 60 - Web Developer Toolbar
 * 1843 - Firebug
 * 8370 - Drupal for Firebug (freezes on install, also compatability issue: http://drupal.org/node/607022)
 */
define('DRUSH_DRUBUNTU_FIREFOX_ADDONS', '60,1843');

/**
 * Firefox extension directory.
 * The string in curly braces is the Firefox application ID (this doesn't change).
 */
define('DRUSH_DRUBUNTU_FIREFOX_EXTENSION_DIR', '/.mozilla/extensions/{ec8030f7-c20a-464f-9b0e-13a3a9e97384}/');

/**
 * Headless Eclipse extension URI.
 */
define('DRUSH_DRUBUNTU_ECLIPSE_HEADLESS_URI', 'http://eclipse.indweb.hu/');

// Do not allow the program to be run as the root user. Ever.
$name = posix_getpwuid(posix_geteuid());
if ($name['name'] == 'root') {
  return drush_set_error('DRUBUNTU_IS_ROOT', dt('You are running the drubuntu script as the root user. Exiting'));
}

/**
 * Implementation of hook_drush_init().
 *
 * This sets the root and/or uri correctly, even when you are in a site
 * directory that is outside of the Drupal root.
 * We do this only if everything looks as we would expect, so we don't
 * interfere with the existing Drush detection logic.
 */
function drubuntu_drush_init() {
  $root = drush_get_option(array('r', 'root'));
  $uri = drush_get_option(array('l', 'uri'));
  if (empty($root) || empty($uri)) {
    $test_path = drush_site_path();
    if (!$test_path) {
      return;
    }
    $test_name = basename($test_path);
    $test_uri = $test_name . '.' . drush_get_option('hostname', 'localhost');
    $test_root = $_SERVER['HOME'] . '/' . drush_get_option('workspace', 'workspace') . '/' . substr($test_name, strrpos($test_name, '.') + 1);
  }
  if (empty($root) && is_dir($test_root) && file_exists($test_root . '/modules/system/system.module')) {    
    drush_set_option('r', $test_root);
  }
  if (empty($uri) && file_exists($test_root . '/sites/' . $test_uri . '/settings.php')) {
    drush_set_option('l', 'http://' . $test_uri);
  }
}

/**
 * Implementation of hook_drush_command().
 */
function drubuntu_drush_command() {
  $items = array();

  $items['drubuntu-install'] = array(
    'description' => "Installs Drubuntu.",
    'arguments' => array(
      'items' => 'A space separated list of install tasks.',
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap at all.
  );
  $items['drubuntu-uninstall'] = array(
    'description' => "Unnstalls Drubuntu.",
    'arguments' => array(
      'items' => 'A space separated list of uninstall tasks.',
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap at all.
  );
  $items['drubuntu-site-add'] = array(
    'description' => "Adds a site.",
    'arguments' => array(
      'name' => 'The name of the site.',
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap at all.
  );
  $items['drubuntu-site-remove'] = array(
    'description' => "Removes a site.",
    'arguments' => array(
      'name' => 'The name of the site.',
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH, // No bootstrap at all.
  );

  return $items;
}

/**
 * Implementation of hook_drush_help().
 */
function drubuntu_drush_help($section) {
  switch ($section) {
    case 'drush:drubuntu-install':
      return dt("Installs Drubuntu - by default everything is installed, but you can also install specific items by passing one of the following arguments:
 lamp (install LAMP)
 eclipse (install Eclipse with PDT and Subclipse)
 webdev (install development tools)
 sandbox (set up an initial sandbox site for testing).");
  }
}

function drush_drubuntu_tasks() {
  return array('lamp', 'eclipse', 'webdev', 'sandbox');
}

function drush_drubuntu_install() {
  $tasks = func_get_args();
  if (empty($tasks)) {
    $tasks = drush_drubuntu_tasks();
  }
  foreach ($tasks as $task) {
    $function = "drush_drubuntu_install_$task";
    if (function_exists($function)) {
      $function();
    }
    else {
      return drush_set_error('DRUBUNTU_TASK_NOT_FOUND', 'Could not find install task ' . $task);
    }
  }
  if (drush_get_option('new_shell', FALSE)) {
    drush_print('User groups changed, starting new shell...');
    shell_exec('su --preserve-environment --command "$(which $SHELL) --login -i" $(whoami)');
    drush_print('NOTE: Drubuntu commands in this shell may not complete correctly.');
  }
  return TRUE;
}

function drush_drubuntu_exec($status, $cmd, $args, $successmessage, $failcode, $failmessage, $ignore_return = FALSE) {
  $password_note = '';
  if (strpos($cmd, 'sudo') !== FALSE) {
    $password_note = ' ' . dt('You may need to enter your password to continue.');
  }
  drush_print(dt($status) . $password_note);
  array_push($args, $cmd);
  if (!call_user_func_array('drush_shell_exec', $args) &&  $ignore_return == FALSE) {
    return drush_set_error($failcode, dt($failmessage) . dt("\nThe specific errors are below:\n!errors", array('!errors' => implode("\n", drush_shell_exec_output()))));
  }
  drush_log(dt($successmessage), 'success');
}

function drush_drubuntu_backup_dir() {
  static $dir;
  if (isset($dir)) {
    return $dir;
  }
  if (!is_dir($_SERVER['HOME'] . '/.drubuntu')) {
    drush_op('mkdir', $_SERVER['HOME'] . '/.drubuntu', 0700);
  }
  if (!is_dir($_SERVER['HOME'] . '/.drubuntu/backups')) {
    drush_op('mkdir', $_SERVER['HOME'] . '/.drubuntu/backups', 0700);
  }
  $dir = $_SERVER['HOME'] . '/.drubuntu/backups/' . date('YmdHis', $_SERVER['REQUEST_TIME']);
  if (!is_dir($dir)) {
    drush_op('mkdir', $dir, 0700);
  }
  return $dir;
}

function drush_drubuntu_sudo_write($destination, $contents) {
  $filename_backup = trim(preg_replace('/[^A-Za-z0-9-_.]/', '_', $destination), '_');
  $filename_code = strtoupper($filename_backup);
  if (file_exists($destination)) {
    // Attempt to backup any existing file.
    $backup_destination = drush_drubuntu_backup_dir() . '/' . $filename_backup;
    // We read the file as sudo, but write it as the current user.
    drush_drubuntu_exec('Backing up file at ' . $destination . ' to ' . $backup_destination . '.', 'sudo cat ' . $destination . ' > ' . $backup_destination, array(), 'File at ' . $destination . ' was backed up successfully.', 'DRUBUNTU_FILE_BACKUP_FAIL_' . $filename_code, 'Unable to backup the file from ' . $destination . ' to ' . $backup_destination . '.');
  }
  $tmpname = tempnam('/tmp', 'drubuntu-write');
  file_put_contents($tmpname, $contents);
  // We copy, rather than move, to ensure correct file ownership.
  // TODO: make this dt()-able.
  drush_drubuntu_exec('Updating/creating file at ' . $destination . '.', 'sudo cp -f ' . $tmpname . ' ' . $destination, array(), 'File at ' . $destination . ' was placed successfully.', 'DRUBUNTU_FILE_PLACE_FAIL_' . $filename_code, 'Problems were encountered placing the file at ' . $destination . '.');
  drush_op('unlink', $tmpname);
}

function drush_drubuntu_install_lamp() {
  $hostname = drush_get_option('hostname', 'localhost');
  drush_drubuntu_exec('Updating Ubuntu software index prior to installs.', 'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq update', array(), 'System update successfull.', 'DRUBUNTU_LAMP_UPDATE_FAILED', 'Problems were encountered updating your system prior to installation.');
  // We use aptitude here as it had better dependency handling for clean removals.
  // Install core LAMP software.
  $cmd = 'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq install ' . drush_get_option('lamp_packages', DRUSH_DRUBUNTU_LAMP_PACKAGES);
  drush_drubuntu_exec('Installing LAMP packages.', $cmd, array(), 'LAMP install successfull.', 'DRUBUNTU_LAMP_INSTALL_FAILED', 'Problems were encountered installing LAMP software.');
  // Install Postfix (preselect local-only delivery to prevent accidentally sending out internet mail to real users on sandbox sites).
  drush_drubuntu_exec('Installing Postfix (for local-only mail delivery).',  'echo postfix postfix/main_mailer_type select Local only | sudo debconf-set-selections && sudo DEBIAN_FRONTEND=noninteractive aptitude -yq install postfix', array(), 'Postfix install successfull.', 'DRUBUNTU_POSTFIX_INSTALL_FAILED', 'Problems were encountered installing Postfix.');

  drush_drubuntu_exec('Adding user to www-data group (to allow access to server created files).', 'sudo adduser $(whoami) www-data', array(), 'User added to www-data group.', 'DRUBUNTU_USER_WWWDATA_GROUP_FAILED', 'Problems were encountered adding the user to www-data group.');
  drush_set_option('new_shell', TRUE);
  
  // Set up PHP configuration.
  $xdebug_directory = drush_get_option('php_xdebug_output_dir', $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/xdebug');
  drush_drubuntu_sudo_write('/etc/php5/conf.d/drubuntu.ini', '
# Below configuration added by Drubuntu.
#
# This file configures PHP and associated modules for Drupal development.

# PHP settings
memory_limit = ' . drush_get_option('php_memory_limit', '128M') . '
upload_max_filesize = ' . drush_get_option('php_upload_max_filesize', '128M') . '
post_max_size = ' . drush_get_option('php_post_max_size', '256M') . '
error_reporting = ' . drush_get_option('php_error_reporting', 'E_ALL') . '

# Xdebug settings
# Set directories, and allow enabling via URL parameters.
xdebug.profiler_output_dir=' . $xdebug_directory . '
xdebug.profiler_enable_trigger=' . drush_get_option('php_xdebug_profiler_enable_trigger', '1') . '
xdebug.trace_output_dir=' . $xdebug_directory . '
xdebug.remote_enable=' . drush_get_option('php_xdebug_remote_enable', '1') . '

# APC settings
# Increase cache size, to prevent fragmentation.
apc.shm_size=' . drush_get_option('php_apc_shm_size', '128') . '
');
  if (!is_dir($xdebug_directory)) {
    drush_op('mkdir', $xdebug_directory, 0770);
    // This shell session will not be a member of the www-data group, so we sudo here.
    drush_shell_exec('sudo chgrp www-data ' . $xdebug_directory);
  }

  // Install PECL modules.
  drush_drubuntu_exec('Installing PECL packages.', 'sudo pecl install -sf ' . drush_get_option('pecl_packages', DRUSH_DRUBUNTU_PECL_PACKAGES), array(), 'PECL install successfull.', 'DRUBUNTU_PECL_INSTALL_FAILED', 'Problems were encountered installing software through PECL.');
  drush_drubuntu_sudo_write('/etc/php5/conf.d/uploadprogress.ini', 'extension=uploadprogress.so');

  // Basic Apache setup.
  drush_drubuntu_exec('Enabling Apache modules.', 'sudo a2enmod ' . drush_get_option('apache_modules', 'vhost_alias rewrite'), array(), 'Apache modules enabled successfull.', 'DRUBUNTU_APACHE_MODULES_ENABLE_FAILED', 'Problems were encountered enabling Apache modules.');
  drush_drubuntu_sudo_write('/etc/apache2/httpd.conf', '
# Below configuration added by Drubuntu.
#
# This file configures Apache so it provides access and .htaccess overrides
# to projects in your workspace and enables virtual hosting.

<Directory "' . $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'">
    Options FollowSymLinks
    AllowOverride All

    Order allow,deny
    Allow from all
</Directory>

ServerName '. $hostname);
  drush_drubuntu_exec('Disabling Apache "default" vhost.', 'sudo a2dissite default', array(), 'Apache "default" vhost disabled.', 'DRUBUNTU_APACHE_DEFAULT_VHOST_DISABLE_FAILED', 'Problems were encountered disabling Apache "default" vhost.');

  // Set up our DNS.
  drush_drubuntu_dns_install();

  // Install our vhost and setup DNS for it.
  drush_drubuntu_install_vhost($hostname);
  drush_drubuntu_dns_add($hostname);

  // User configuration.
  drush_drubuntu_exec('Symlinking the Apache log directory to workspace for easy reference.', 'ln -sfT /var/log/apache2 ' . $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/logs', array(), 'Apache log directory symlinked to workspace successfully.', 'DRUBUNTU_APACHE_LOG_SYMLINK_FAILED', 'Problems were encountered symlinking the Apache log directory to workspace.');

  // Add global drubuntu.settings.inc if it doesn't exist.
  if (!file_exists($_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/drubuntu.settings.inc')) {
    file_put_contents($_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/drubuntu.settings.inc', '<?php
/**
 * @file
 * Drubuntu global configuration file.
 *
 * Configuration added here will apply to all sites (including across all core
 * installs). This can be a useful place to apply generic development settings.
 * Some examples follow:
 */
# $conf[\'preprocess_css\'] = FALSE;
# $conf[\'preprocess_js\'] = FALSE;
# $conf[\'securepages_enable\'] = FALSE;
# $conf[\'devel_rebuild_theme_registry\'] = TRUE;
');
  }

  // Add my.cnf if it doesn't exist.
  if (!file_exists($_SERVER['HOME'] . '.my.cnf')) {
    $my_cnf = "[client]\n";
    foreach (array('user' => 'root', 'password' => '', 'host' => '', 'port' => '') as $parameter => $value) {
      if (!empty($value)) {
        $my_cnf .= $parameter . '=' . drush_get_option('db_' . $parameter, $value) . "\n";
      } 
    }
    file_put_contents($_SERVER['HOME'] . '/.my.cnf', $my_cnf);
  }

  drush_drubuntu_exec('Restarting Apache.', 'sudo service apache2 restart', array(), 'Apache restart successfull.', 'DRUBUNTU_APACHE_RESTART_FAILED', 'Problems were encountered restarting Apache.');
  drush_drubuntu_exec('Restarting Dnsmasq.', 'sudo service dnsmasq restart', array(), 'Dnsmasq restart successfull.', 'DRUBUNTU_DNSMASQ_RESTART_FAILED', 'Problems were encountered restarting Dnsmasq.');
}

function drush_drubuntu_dns_install() {
  // Set up Dnsmasq - ensure we listen only on the "lo" interface.
  $dnsmasq_conf = file_get_contents('/etc/dnsmasq.conf');
  $dnsmasq_conf = str_replace('#interface=', 'interface=lo', $dnsmasq_conf);
  drush_drubuntu_sudo_write('/etc/dnsmasq.conf', $dnsmasq_conf);
  // Add to dhclient for users with dynamic IPs.
  $dhclient = file_get_contents('/etc/dhcp3/dhclient.conf');
  $dhclient = str_replace('#prepend domain-name-servers 127.0.0.1;', 'prepend domain-name-servers 127.0.0.1;', $dhclient);
  drush_drubuntu_sudo_write('/etc/dhcp3/dhclient.conf', $dhclient);
  // Set up resolv.conf for users with static IPs.
  $resolv_conf = file_get_contents('/etc/resolv.conf');
  // Ensure the local nameserver is above any others.
  $ns = "nameserver 127.0.0.1\n";
  $resolv_conf = str_replace($ns, '', $resolv_conf);
  $resolv_conf = preg_replace('/nameserver/', $ns . 'nameserver', $resolv_conf, 1, $count);
  if (!$count) {
    // No existing nameservers, so let's just add ours at the bottom of the file.
    $resolv_conf .= $ns; 
  }
  drush_drubuntu_sudo_write('/etc/resolv.conf', $resolv_conf);
}

function drush_drubuntu_dns_uninstall() {
  // Remove from dhclient.
  $dhclient = file_get_contents('/etc/dhcp3/dhclient.conf');
  $dhclient = str_replace('prepend domain-name-servers 127.0.0.1;', '#prepend domain-name-servers 127.0.0.1;', $dhclient);
  drush_drubuntu_sudo_write('/etc/dhcp3/dhclient.conf', $dhclient);
  // Remove local nameserver from resolv.conf.
  $resolv_conf = file_get_contents('/etc/resolv.conf');
  $ns = "nameserver 127.0.0.1\n";
  $resolv_conf = str_replace($ns, '', $resolv_conf);
  drush_drubuntu_sudo_write('/etc/resolv.conf', $resolv_conf);
}

function drush_drubuntu_dns_add($tld) {
  $dnsmasq_conf = file_get_contents('/etc/dnsmasq.conf');
  // Insert our line if needed.
  $line = 'address=/.' . $tld . '/127.0.0.1';
  if (strpos($dnsmasq_conf, $line) === FALSE) {
    $dnsmasq_conf .= $line . "\n";
  }
  drush_drubuntu_sudo_write('/etc/dnsmasq.conf', $dnsmasq_conf);
}

function drush_drubuntu_install_vhost($tld) {
  drush_drubuntu_sudo_write('/etc/apache2/sites-available/drubuntu', '
# Below configuration added by Drubuntu.
#
# This file configures Apache so that when you browse to
# http://PLATFORM.' . $tld . '/ or http://SITE.PLATFORM.' . $tld . '/ 
# Apache will use the docroot ' . $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/PLATFORM/.

<VirtualHost *:80>
    ServerName ' . $tld . '
    ServerAlias *.' . $tld . '
    <IfModule mod_rewrite.c>
        RewriteEngine On

        # We set the workspace location centrally here.
        RewriteRule ^.*$ - [E=workspace:' . $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .']
        
        # Ensure the hostname is lower case.
        RewriteMap lowercase int:tolower

        # These rewrite rules detect a Drupal installation and adds rewrite
        # rules for clean URLs for dynamic vhosts (since the default .htaccess
        # does not work for dynamic vhosts).
        RewriteCond %{HTTP_HOST} ^([^.]*\.)?([^.]+)\.' . $tld . '
        RewriteCond %{ENV:workspace}/%2/modules/system/system.module -f
        RewriteCond %{ENV:workspace}/%2%{REQUEST_FILENAME} !-f
        RewriteCond %{ENV:workspace}/%2%{REQUEST_FILENAME} !-d
        RewriteRule ^(.*)$ %{ENV:workspace}/%2/index.php?q=$1 [L,QSA]

        # These rules rewrite requests for other files or scripts.
        RewriteCond %{HTTP_HOST} ^([^.]*\.)?([^.]+)\.' . $tld . '
        RewriteRule ^/(.*)$ %{ENV:workspace}/%2/$1 [L]
    </IfModule>
</VirtualHost>
');
  drush_drubuntu_exec('Enabling Apache vhost.', 'sudo a2ensite drubuntu', array(), 'Apache vhost enabled.', 'DRUBUNTU_APACHE_VHOST_ENABLE_FAILED', 'Problems were encountered enabling Apache vhost.');
}

function drush_drubuntu_install_drubuntu_settings_inc($platform) {
  $infile = $outfile = $platform . '/sites/drubuntu.settings.inc';
  if (!file_exists($infile)) {
    $infile = $platform . '/sites/default/default.settings.php';
  }
  if (!file_exists($infile)) {
    $infile = $platform . '/sites/default/settings.php';
  }
  if (!file_exists($infile)) {
    return drush_set_error('DRUBUNTU_SETTINGS_PHP_TEMPLATE_NOT_FOUND', 'A suitable template for settings.php could not be found.');
  }
  $drubuntu_settings_inc = file_get_contents($infile);
  $search = $replace = array();
  // Add some explanation on the purpose of this file.
  $search[0] = '/^ \* Drupal site-specific configuration file\.$/m';
  $replace[0] = ' * Drubuntu multisite configuration file.
 * 
 * Configuration added here will affect all sites in this multisite install.
 * Each site settings.php includes this file.
 * This file also includes the global ~/'. drush_get_option('workspace', 'workspace') .'/drubuntu.settings.inc.
 * $db_url and $conf[\'file_directory_path\'] are dynamically determined here.';
  // Add some code to dynamically determine the site database.
  $search[1] = '/^(\$drubuntu_site.*\n)*\$db_url.*$/m';
  $replace[1] = '$drubuntu_site = basename(conf_path());
$db_url = \'' .
drush_get_option('db_type', 'mysql') . '://' .
drush_get_option('db_user', 'root') . ':' .
drush_get_option('db_passwd', '') . '@' .
drush_get_option('db_host', 'localhost') .
'/\' . str_replace(\'.\', \'_\', $drubuntu_site);';

  // Add some code to dynamically determine the base_url.
  $search[2] = '/^(# )?\$base_url = .*$/m';
  $replace[2] = "// Workaround for https://issues.apache.org/bugzilla/show_bug.cgi?id=40102
\$base_root = (isset(\$_SERVER['HTTPS']) && \$_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
\$base_url = \$base_root .= '://'. \$_SERVER['SERVER_NAME'];";
  
  // Remove the 2 lines we add below.
  $search[3] = '/^\$conf\[\\\'file_directory_path\\\'\].*\n/m';
  $replace[3] = '';
  $search[4] = '/^include_once.*drubuntu\.settings\.inc.*\n/m';
  $replace[4] = '';    $drubuntu_settings_inc = preg_replace($search, $replace, $drubuntu_settings_inc);
  $drubuntu_settings_inc .= '$conf[\'file_directory_path\'] = \'sites/\' . $drubuntu_site . \'/files\';' . "\n";
  $drubuntu_settings_inc .= 'include_once(\'' . $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/drubuntu.settings.inc\');' . "\n";
  if (file_exists($outfile)) {
    // Attempt to backup any existing file.
    drush_op('copy', $outfile, drush_drubuntu_backup_dir() . '/drubuntu.settings.inc');
  }
  file_put_contents($outfile, $drubuntu_settings_inc);
}

function drush_drubuntu_install_eclipse() {
  // Check the user accepts the JRE licence. If so then we run the fix_db.pl
  // script (the debconf database gets inconsistent after a purge), pre-seed
  // the database with an accepted value, and install as normal.
  drush_confirm("Please read the JRE licence available at http://www.java.com/en/download/license.jsp\nDo you accept this licence:");
  drush_drubuntu_exec('Installing the JRE.',  'echo sun-java6-jre shared/accepted-sun-dlj-v1-1 boolean true | sudo debconf-set-selections && sudo aptitude -yq install sun-java6-jre', array(), 'JRE install successfull.', 'DRUBUNTU_JRE_INSTALL_FAILED', 'Problems were encountered installing JRE.');
  drush_drubuntu_exec('Adding apt-get repository for Eclipse plugins.',  'sudo add-apt-repository ppa:yogarine/eclipse/ubuntu && sudo apt-get update', array(), 'Repository added successfully.', 'DRUBUNTU_ECLIPSE_REPO_FAILED', 'Problems adding the repository.');
  drush_drubuntu_exec('Installing Eclipse and plugins.',  'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq install ' . DRUSH_DRUBUNTU_ECLIPSE_PACKAGES, array(), 'Eclipse and plugins installed successfully.', 'DRUBUNTU_ECLIPSE_INSTALL_FAILED', 'Problems were encountered installing Eclipse and plugins.');

  drush_print('Installing Eclipse configuration.');
  if (is_dir($_SERVER['HOME'] . '/.eclipse')) {
    // Move any existing .eclipse directory out of the way, since we move things
    // around quite a bit.
    drush_op('rename', $_SERVER['HOME'] . '/.eclipse', drush_drubuntu_backup_dir() . '/.eclipse');
  }
  $workspace = $_SERVER['HOME'] . '/' . drush_get_option('workspace', 'workspace');
  drush_shell_exec('mkdir -p ' . $workspace . '/.metadata/.plugins/org.eclipse.core.runtime/.settings/');
  $twospaces = array(
    'indentationChar=space',
    'indentationSize=2',
  );
  $files_configs = array(
    'org.eclipse.core.runtime.prefs' => array(
      // Set content type associations.
      'content-types/org.eclipse.php.core.phpsource/file-extensions=module,engine,theme,profile,install,test',
      'org.eclipse.core.runtime.prefs:content-types/org.eclipse.wst.html.core.htmlsource/file-extensions=xtmpl',
    ),
    'org.eclipse.php.core.prefs' => array(
      // Two spaces instead of tabs.
      'org.eclipse.php.core.phpForamtterUseTabs=false',
      'org.eclipse.php.core.phpForamtterIndentationSize=2',
      // Xdebug configuration.
      'org.eclipse.php.debug.coreinstalledPHPInis=/etc/php5/apache2/php.ini',
      'org.eclipse.php.debug.coreinstalledPHPDebuggers=org.eclipse.php.debug.core.xdebugDebugger',
      'org.eclipse.php.debug.coreinstalledPHPDefaults=org.eclipse.php.debug.core.xdebugDebugger\=PHP',
      'org.eclipse.php.debug.coreinstalledPHPLocations=/usr/bin/php',
      'org.eclipse.php.debug.coreinstalledPHPNames=PHP',
      'org.eclipse.php.debug.corephp_debugger_id=org.eclipse.php.debug.core.xdebugDebugger',
    ),
    'org.eclipse.php.ui.prefs' => array(
      // Add "Id" for new files.
      // TODO: Incorporate hook templates here (store as separate XML file and quote).
      'org.eclipse.php.ui.editor.templates=<?xml version\="1.0" encoding\="UTF-8" standalone\="no"?><templates><template autoinsert\="false" context\="newPhp" deleted\="false" description\="Simple php file" enabled\="true" id\="org.eclipse.php.ui.editor.templates.newPhp.author" name\="New simple PHP file">&lt;?php\n// $$Id: drubuntu.drush.inc,v 1.1 2010/03/12 01:24:06 grugnog Exp $$\n${cursor}</template></templates>',
    ),
    'org.eclipse.ui.editors.prefs' => array(
      // Two spaces instead of tabs.
      'spacesForTabs=true',
      'tabWidth=2',
      'printMargin=true',
    ),
    // Two spaces instead of tabs.
    'org.eclipse.wst.css.core.prefs' => $twospaces,
    'org.eclipse.wst.css.ui.prefs' => array(
      // Add "Id" for new files.
      'org.eclipse.wst.sse.ui.custom_templates=<?xml version\="1.0" encoding\="UTF-8" standalone\="no"?><templates><template autoinsert\="true" context\="css_new" deleted\="false" description\="new css file" enabled\="true" id\="org.eclipse.wst.css.ui.internal.templates.newcss" name\="New CSS File">/* $$Id: drubuntu.drush.inc,v 1.1 2010/03/12 01:24:06 grugnog Exp $$ */\n@CHARSET "${encoding}";</template></templates>',
    ),
    'org.eclipse.wst.html.ui.prefs' => array(
      // Disable autoactivate completion (slow), use ctrl-return instead.
      'autoPropose=false',
    ),
    // Two spaces instead of tabs.
    'org.eclipse.wst.html.core.prefs' => $twospaces,
    'org.eclipse.wst.jsdt.core.prefs' => array(
      // Two spaces instead of tabs.
      'org.eclipse.wst.jsdt.core.formatter.tabulation.size=2',
      'org.eclipse.wst.jsdt.core.formatter.indentation.size=2',
      'org.eclipse.wst.jsdt.core.formatter.tabulation.char=space',
    ),
    'org.eclipse.wst.jsdt.ui.prefs' => array(
      // Add "Id" for new files.
      'org.eclipse.wst.jsdt.ui.text.custom_code_templates=<?xml version\="1.0" encoding\="UTF-8" standalone\="no"?><templates><template autoinsert\="false" context\="newtype_context" deleted\="false" description\="Newly created files" enabled\="true" id\="org.eclipse.wst.jsdt.ui.text.codetemplates.newtype" name\="newtype">// $$Id: drubuntu.drush.inc,v 1.1 2010/03/12 01:24:06 grugnog Exp $$\n${filecomment}\n${package_declaration}\n\n${typecomment}\n${type_declaration}</template></templates>',
      // Disable autoactivate completion (slow), use ctrl-return instead.
      'content_assist_autoactivation=false',
    ),
    // Two spaces instead of tabs.
    'org.eclipse.wst.xml.core.prefs' => $twospaces,
  );
  // This takes the above array and merges it with the existing configuration,
  // or adds it if there is no existing configuration.
  foreach ($files_configs as $filename => $configs) {
    $filename = $workspace . '/.metadata/.plugins/org.eclipse.core.runtime/.settings/' . $filename;
    $file = '';
    if (file_exists($filename)) {
      $file = file_get_contents($filename);
    }
    foreach ($configs as $config) {
      $key = substr($config, 0, strpos($config, '='));
      $file = preg_replace('/' . preg_quote($key) . '.*/', $config, $file, -1 , $count);
      if ($count == 0) {
       $file .= $config . "\n";
      }
    }
    file_put_contents($filename, $file);
  }

  // Install HeadlessEclipse, so we can automatically import priojects.
  // TODO: Look at changing this to use http://help.eclipse.org/galileo/topic/org.eclipse.platform.doc.isv/guide/p2_director.html
  drush_shell_exec('java -jar /usr/lib/eclipse/plugins/org.eclipse.equinox.launcher_*.jar -application org.eclipse.update.core.standaloneUpdate -command search -from ' . DRUSH_DRUBUNTU_ECLIPSE_HEADLESS_URI);
  preg_match('/^ *"([^"]*)" ([^ "]*\.headlesseclipse) ([^ "]*)$/m', implode("\n", drush_shell_exec_output()), $matches);
  if (!empty($matches)) {
    drush_drubuntu_exec('Installing HeadlessEclipse, so we can automatically import projects.',  'java -jar /usr/lib/eclipse/plugins/org.eclipse.equinox.launcher_*.jar -application org.eclipse.update.core.standaloneUpdate -command install -featureId ' . $matches[2] . ' -version ' .  $matches[3] . ' -from ' . DRUSH_DRUBUNTU_ECLIPSE_HEADLESS_URI . ' -to ' . $_SERVER['HOME'] . '/.eclipse/org.eclipse.platform_3*/', array(), 'HeadlessEclipse install successfull.', 'DRUBUNTU_HEADLESSECLIPSE_INSTALL_FAILED', 'Problems were encountered installing HeadlessEclipse.');
  }
  else {
    drush_set_error('DRUBUNTU_HEADLESSECLIPSE_FEATURE_NOT_FOUND', 'HeadlessEclipse could not be located on the update site');
  }

  // Add Drush, Provision and Drubuntu projects (patches welcome...hint, hint).
  drubuntu_eclipse_add_project(array($workspace . '/drush', $workspace . '/provision', $workspace . '/drubuntu'));

  // Copy icon to Desktop, since this is a key application.
  drush_op('copy', '/usr/share/applications/eclipse.desktop', $_SERVER['HOME'] . '/Desktop/eclipse.desktop');
  drush_op('chmod', $_SERVER['HOME'] . '/Desktop/eclipse.desktop', 0755);

  drush_log(dt('Eclipse configuration installed.'), 'success');
}

function drush_drubuntu_install_webdev() {
  // Install core software.
  drush_drubuntu_exec('Adding Google Chrome repository.', "echo 'deb http://dl.google.com/linux/deb/ stable main' | sudo tee /etc/apt/sources.list.d/google-chrome.list && sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys A040830F7FAC5991 && sudo DEBIAN_FRONTEND=noninteractive aptitude -yq update", array(), 'Google Chrome repository added successfull.', 'DRUBUNTU_CHROME_REPO_FAILED', 'Problems were encountered adding the Google Chrome repository.');
  drush_drubuntu_exec('Installing web development packages.', 'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq install ' . drush_get_option('webdev_packages', DRUSH_DRUBUNTU_WEBDEV_PACKAGES), array(), 'Web development packages were installed successfull.', 'DRUBUNTU_LAMP_INSTALL_FAILED', 'Problems were encountered installing web development packages.');
  drush_drubuntu_exec('Symlinking phpmyadmin to workspace (http://phpmyadmin.'. drush_get_option('hostname', 'localhost') .'/).', 'ln -sfT /usr/share/phpmyadmin ' . $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') .'/phpmyadmin', array(), 'Phpmyadmin symlinked to workspace.', 'DRUBUNTU_PHPMYADMIN_SYMLINK_FAILED', 'Problems were encountered symlinking phpmyadmin to workspace.');

  // Install IEs4Linux for cross-browser testing.
  $uri = drush_get_option('ies4linux_uri', DRUSH_DRUBUNTU_IES4LINUX_URI);
  $archive = '/tmp/drubuntu-ies4linux.tar.gz';
  drush_drubuntu_exec('Downloading IEs4Linux.', 'wget -q -nc -c --timeout=30 -O' . $archive . " '" . $uri . "'", array(), 'IEs4Linux downloaded.', 'DRUBUNTU_IES4LINUX_DOWNLOAD_FAILED', 'Problems were encountered downloading IEs4Linux.');
  drush_drubuntu_exec('Extracting IEs4Linux.', 'tar --directory=' . $_SERVER['HOME'] . ' -xzvf ' . $archive, array(), 'IEs4Linux extracted.', 'DRUBUNTU_IES4LINUX_EXTRACT_FAILED', 'Problems were encountered extracting IEs4Linux.');
  drush_drubuntu_exec('Installing IEs4Linux.', $_SERVER['HOME'] . '/ies4linux-*/ies4linux --no-gui --beta-install-ie7', array(), 'IEs4Linux installed.', 'DRUBUNTU_IES4LINUX_INSTALL_FAILED', 'Problems were encountered installing IEs4Linux.', TRUE);
  drush_op('unlink', $archive);
  // Chmod the Desktop icons so they work normally.
  drush_op('chmod', $_SERVER['HOME'] . '/Desktop/ies4linux-ie6.desktop', 0755);
  drush_op('chmod', $_SERVER['HOME'] . '/Desktop/ies4linux-ie7.desktop', 0755);

  // Install Firefox web development Add-ons.
  $packages = explode(',', drush_get_option('firefox_addons', DRUSH_DRUBUNTU_FIREFOX_ADDONS));
  $uris = array();
  foreach ($packages as $package) {
    $uris[] = sprintf('https://addons.mozilla.org/en-US/firefox/downloads/latest/%1$d/addon-%1$d-latest.xpi', $package);
  }
  drush_drubuntu_exec('Installing Firefox web development Add-ons.', "wget -q -nc -c --timeout=30 --directory-prefix='" . $_SERVER['HOME'] . DRUSH_DRUBUNTU_FIREFOX_EXTENSION_DIR . "' '" . implode("' '", $uris) . "'", array(), 'Firefox web development Add-ons installed.', 'DRUBUNTU_FIREFOX_ADDONS_INSTALL_FAILED', 'Problems were encountered installing Firefox web development Add-ons.');
  // Chrome likes to steal the default browser, so we set it back to Firefox here.
  drush_drubuntu_exec('Setting default browser back to Firefox.', 'sudo update-alternatives --set gnome-www-browser /usr/bin/firefox', array(), 'Default browser set back to Firefox.', 'DRUBUNTU_DEFAULT_BROWSER_SET', 'Problems were encountered setting default browser back to Firefox.');
  drush_drubuntu_exec('Setting default browser back to Firefox.', 'sudo update-alternatives --set x-www-browser /usr/bin/firefox', array(), 'Default browser set back to Firefox.', 'DRUBUNTU_DEFAULT_BROWSER_SET', 'Problems were encountered setting default browser back to Firefox.');
}

function drush_drubuntu_install_sandbox() {
  $sandbox_platforms = explode(',', drush_get_option('sandbox_platforms', 'drupal'));
  foreach ($sandbox_platforms as $platform) {
    $data = array('install-platform' => $platform);
    drush_backend_invoke_args('drubuntu-site-add', array(drush_get_option('sandbox_name', 'sandbox') . '.' . $platform), $data);
  }
}

function drush_drubuntu_uninstall() {
  $op = 'remove';
  if (drush_get_option('purge', FALSE)) {
    $op = 'purge';
  }

  $tasks = func_get_args();
  if (empty($tasks)) {
    $tasks = drush_drubuntu_tasks();
  }
  $tasks = array_reverse($tasks);
  foreach ($tasks as $task) {
    $function = "drush_drubuntu_uninstall_$task";
    $function($op);
  }
}


function drush_drubuntu_uninstall_lamp($op) {
  drush_drubuntu_exec('Disabling Apache vhost.', 'sudo a2dissite drubuntu', array(), 'Apache vhost disabled.', 'DRUBUNTU_APACHE_VHOST_DISABLE_FAILED', 'Problems were encountered disabling Apache vhost.');
  if ($op == 'purge') {
    drush_drubuntu_exec('Removing Apache vhost.', 'sudo rm -f /etc/apache2/sites-available/drubuntu', array(), 'Apache vhost removed.', 'DRUBUNTU_APACHE_VHOST_REMOVE_FAILED', 'Problems were encountered removing Apache multisite vhost.');
  }
  drush_drubuntu_exec('Uninstalling PECL packages.', 'sudo pecl uninstall ' . drush_get_option('pecl_packages', DRUSH_DRUBUNTU_PECL_PACKAGES), array(), 'PECL uninstall successfull.', 'DRUBUNTU_PECL_UNINSTALL_FAILED', 'Problems were encountered uninstalling software through PECL.');
  $cmd = 'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq ' . $op . ' ' . drush_get_option('lamp_packages', DRUSH_DRUBUNTU_LAMP_PACKAGES);
  drush_drubuntu_exec('Uninstalling LAMP packages.', $cmd, array(), 'LAMP uninstall successfull.', 'DRUBUNTU_LAMP_UNINSTALL_FAILED', 'Problems were encountered uninstalling LAMP software.');
  drush_drubuntu_dns_uninstall();
  if ($op == 'purge') {
    $workspace = $_SERVER['HOME'] . '/' . drush_get_option('workspace', 'workspace');
    $cmd = 'sudo mv ' .
      $workspace . '/logs ' .
      $workspace . '/phpmyadmin ' .
      $workspace . '/xdebug ' .
      $workspace . '/drubuntu.settings.inc ' .
      $_SERVER['HOME'] . '/.my.cnf ' .
      '/etc/php5 ' . drush_drubuntu_backup_dir();
      drush_drubuntu_exec('Cleaning up files.', $cmd, array(), 'Files cleaned up.', 'DRUBUNTU_FILE_CLEANUP_FAILED', 'Problems were encountered cleaning up files.');
  }
}

function drush_drubuntu_uninstall_eclipse($op) {
  drush_drubuntu_exec('Uninstalling Eclipse and plugins.',  'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq ' . $op . ' ' . DRUSH_DRUBUNTU_ECLIPSE_PACKAGES, array(), 'Eclipse and plugins uninstalled successfully.', 'DRUBUNTU_ECLIPSE_UNINSTALL_FAILED', 'Problems were encountered uninstalling Eclipse and plugins.');
}

function drush_drubuntu_uninstall_webdev($op) {
  // Note that Google Chrome repository sometimes generates a 2 minute wait.
  drush_drubuntu_exec('Uninstalling web development packages.', 'sudo DEBIAN_FRONTEND=noninteractive aptitude -yq ' . $op . ' ' . drush_get_option('webdev_packages', DRUSH_DRUBUNTU_WEBDEV_PACKAGES), array(), 'Web development packages were uninstalled successfull.', 'DRUBUNTU_LAMP_UNINSTALL_FAILED', 'Problems were encountered uninstalling web development packages.');
  drush_drubuntu_exec('Uninstalling IEs4Linux.', 'mv ' . $_SERVER['HOME'] . '/.ies4linux ' . $_SERVER['HOME'] . '/ies4linux-* ' . drush_drubuntu_backup_dir(), array(), 'IEs4Linux was uninstalled successfull.', 'DRUBUNTU_IES4LINUX_UNINSTALL_FAILED', 'Problems were encountered uninstalling IEs4Linux.');
  // There is no easy/safe way to uninstall Firefox Add-ins.
}

function drush_drubuntu_uninstall_sandbox($op) {
  if ($op == 'purge') {
    $sandbox_platforms = explode(',', drush_get_option('sandbox_platforms', 'drupal'));
    foreach ($sandbox_platforms as $platform) {
      drush_drubuntu_site_remove(drush_get_option('sandbox_name', 'sandbox') . '.' . $platform);
    }
  }
}

function drush_drubuntu_db_query($query) {
  $pass = '';
  if (drush_get_option('db_password', '') !== '') {
    $pass = ' -p' . drush_get_option('db_password', '');
  }
  drush_shell_exec('mysql -u' . drush_get_option('db_user', 'root') . $pass . ' -h' . drush_get_option('db_host', 'localhost') . ' -h' . drush_get_option('db_port', '0') . " -e'" . $query . "'");
}

function drush_drubuntu_install_platform($platform) {
  $install_platform = drush_get_option('install-platform', 'drupal');
  $oldcwd = getcwd();
  $landing = $_SERVER['HOME'] . '/.drubuntu/landing';
  if (!is_dir($landing)) {
    drush_op('mkdir', $landing);
  }
  // Backup any existing directories.
  if (is_dir($platform)) {
    drush_op('rename', $platform, drush_drubuntu_backup_dir() . '/' . $name);
  }
  if (is_dir($landing . '/drupal')) {
    drush_op('rename', $landing, drush_drubuntu_backup_dir() . '/landing');
  }
  drush_op('chdir', $landing);
  $data = array('destination' => $landing);
  drush_backend_invoke_args('pm-download', array($install_platform), $data);
  drush_shell_exec('mv ' . $landing . '/drupal* ' . $platform);
  drush_shell_exec('rm -rf ' . $landing);
  drush_op('chdir', $platform);
  if (!is_dir($platform)) {
    return drush_set_error('DRUBUNTU_CORE_DOWNLOAD_FAILED', dt('Drupal core could not be downloaded successfully.'));
  }
  drush_drubuntu_install_drubuntu_settings_inc($platform);
  return TRUE;
}

function drush_drubuntu_get_platform($name, $create) {
  if (empty($name)) {
    return drush_set_error('DRUBUNTU_SITE_ADD_NO_NAME', dt('Invalid site name.'));
  }
  $workspace = drush_get_option('workspace', 'workspace');
  $site = array();
  // Allow users to use the full DNS name if they like.
  $name = str_replace('.'. drush_get_option('hostname', 'localhost'), '', $name);
  $platform = drush_locate_root();
  $site_name = $name;
  // If the user specified a platform, change to that directory first.
  $dot = strrpos($name, '.');
  if ($dot !== FALSE) {
    $platform = $_SERVER['HOME'] . '/'. $workspace .'/' . substr($name, $dot + 1);
    $site_name = substr($name, 0, $dot);
    if (is_dir($platform)) {
      drush_op('chdir', $platform);
    }
  }

  // If we don't have a valid platform, then create one.
  if (!drush_valid_drupal_root($platform)) {
    if (empty($platform)) {
      // We didn't have a site.platform syntax passed in, so we assume
      // that the name passed in is the desired platform name.
      $platform = $_SERVER['HOME'] . '/'. $workspace .'/' . $name;
      $site_name = 'default';
    }
    if ($create && !drush_drubuntu_install_platform($platform)) {
      return FALSE;
    }
  }

  $uri = $site_name . '.' . basename($platform) . '.' . drush_get_option('hostname', 'localhost');
  // Setup the environment for progressive bootstrap.
  drush_set_option('root', $platform);
  drush_set_option('uri', $uri);
  $_SERVER['SERVER_NAME'] = $uri;
  if (drush_bootstrap_validate(DRUSH_BOOTSTRAP_DRUPAL_ROOT)) {
    drush_bootstrap(DRUSH_BOOTSTRAP_DRUPAL_ROOT);
    $site['root'] = $platform;
    $site['site'] = $site_name;
    $site['core'] = drush_get_context('DRUSH_DRUPAL_MAJOR_VERSION');
    $site['uri'] = $uri;
    $site['site_directory'] = drush_get_context('DRUSH_DRUPAL_ROOT') . '/sites/' . $uri;
    $site['workspace_directory'] = $_SERVER['HOME'] . '/'. $workspace .'/' . $site_name . '.' . basename($platform);
    $site['database'] = str_replace('.', '_', $uri);
    return $site;
  }
  return drush_set_error('DRUBUNTU_SITE_ADD_NO_ROOT', dt('Could not find a valid Drupal root to work on.'));
}

function drubuntu_eclipse_add_project($directories) {
  $staging = $_SERVER['HOME'] . '/.drubuntu/staging';
  if (!is_dir($staging)) {
    drush_op('mkdir', $staging);
  }
  drush_op('symlink', $_SERVER['HOME'] . '/'. drush_get_option('workspace', 'workspace') . '/.metadata', $staging . '/.metadata');
  foreach ($directories as $directory) {
    file_put_contents($directory . '/.project', '<?xml version="1.0" encoding="UTF-8"?>
<projectDescription>
<name>' . $directory . '</name><comment></comment><projects></projects>
<buildSpec>
<buildCommand><name>org.eclipse.wst.validation.validationbuilder</name><arguments></arguments></buildCommand>
<buildCommand><name>org.eclipse.dltk.core.scriptbuilder</name><arguments></arguments></buildCommand>
</buildSpec>
<natures><nature>org.eclipse.php.core.PHPNature</nature></natures>
</projectDescription>');
    drush_op('symlink', $directory, $staging . '/' . basename($directory));
  }
  drush_shell_exec('eclipse -nosplash -data ' . $staging . ' -application com.ind.eclipse.headlessworkspace.Application import');
  drush_shell_exec('rm -rf ' . $staging);
}

function drush_drubuntu_site_add($name) {
  $site = drush_drubuntu_get_platform($name, TRUE);
  if (file_exists($site['site_directory'] . '/settings.php')) {
    drush_log('Site ' . $site['uri'] . ' appears to already exist');
    return;
  }
  drush_op('mkdir', $site['workspace_directory'], 0755);
  drush_op('symlink', $site['workspace_directory'], $site['site_directory']);
  drush_op('mkdir', $site['site_directory'] . '/files', 0775);
  drush_op('chmod', $site['site_directory'] . '/files', 0775);
  $sudo = '';
  if (drush_get_option('new_shell', FALSE)) {
    // We are just installing, and this session doesn't have the www-data group
    // yes, so we need to sudo this.
    $sudo = 'sudo ';
  }
  drush_shell_exec($sudo . 'chgrp www-data ' . $site['site_directory'] . '/files');
  file_put_contents($site['site_directory'] . '/settings.php', '<?php
/**
 * @file
 * Site specific configuration file.
 *
 * $db_url and other general configurations are determined dynamically in
 * sites/drubuntu.settings.inc.
 * This file is a good place for any site specific configuration or overrides.
 */
include_once(\'./sites/drubuntu.settings.inc\');
');

  drush_drubuntu_db_query('CREATE DATABASE ' . $site['database']);
  drush_set_option('uri' , 'http://' . $site['uri']);
  drush_set_option('site_url', $site['uri']);
  drush_set_option('profile', 'default');
  drush_set_option('language', 'en');
  drush_set_option('client_email', drush_get_option('client_email'), 'site');
  drush_set_option('site_port', 80);
  include_once($_SERVER['HOME'] . '/.drush/provision/platform/install.provision.inc');
  drush_include_engine('drupal', 'install', $site['core']);
  $link = 'http://' . $site['uri'];
  if (variable_get('install_task', '') == 'done') {
    $link = drush_get_option('login_link');
  }
  // Start browser in the background.
  shell_exec("sensible-browser '" . $link . "' > /dev/null 2>/dev/null &");

  // Add a project in Eclipse.
  drubuntu_eclipse_add_project(array($site['root'], $site['workspace_directory']));
}

function drush_drubuntu_site_remove($name) {
  $site = drush_drubuntu_get_platform($name, FALSE);
  if (!file_exists($site['site_directory'] . '/settings.php')) {
    drush_log('Site ' . $site['uri'] . ' does not appear to exist');
    return;
  }

  // Back up and drop the database.
  drush_bootstrap(DRUSH_BOOTSTRAP_DRUPAL_CONFIGURATION);
  drush_set_option('result-file', drush_drubuntu_backup_dir() . '/' . $site['site'] . '.sql');
  drush_sql_dump_execute();
  drush_drubuntu_db_query('DROP DATABASE ' . $site['database']);

  // Remove site code.
  drush_shell_exec('mv ' . $site['site_directory'] . ' ' . drush_drubuntu_backup_dir());
  drush_shell_exec('mv ' . $site['workspace_directory'] . ' ' . drush_drubuntu_backup_dir());

  // If we just removed it's last site we can remove (backup) Drupal core also.
  $sites = $site['root'] . '/sites';
  if (is_dir($sites)) {
    $files = drush_scan_directory($sites, '/^settings\.php$/');
    if (empty($files)) {
      drush_shell_exec('mv ' . $site['root'] . ' ' . drush_drubuntu_backup_dir());
    }
  }
}
